"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = getSchema;
const grab_host_names_1 = __importDefault(require("../utils/grab-host-names"));
/**
 * # Get Schema for Database, table, or field *
 */
function getSchema(_a) {
    return __awaiter(this, arguments, void 0, function* ({ key, database, field, table, user_id, }) {
        const grabedHostNames = (0, grab_host_names_1.default)();
        const { host, port, scheme } = grabedHostNames;
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        const httpResponse = yield new Promise((resolve, reject) => {
            const queryObject = { database, field, table };
            let query = Object.keys(queryObject)
                .filter((k) => queryObject[k])
                .map((k) => `${k}=${queryObject[k]}`)
                .join("&");
            scheme
                .request({
                method: "GET",
                headers: {
                    "Content-Type": "application/json",
                    Authorization: key ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                },
                port,
                hostname: host,
                path: `/api/query/${user_id || grabedHostNames.user_id}/get-schema` + ((query === null || query === void 0 ? void 0 : query.match(/./)) ? `?${query}` : ""),
            }, 
            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("end", function () {
                    resolve(JSON.parse(str));
                });
                response.on("error", (err) => {
                    resolve(null);
                });
            })
                .end();
        });
        return httpResponse;
    });
}
