"use strict";
// @ts-check
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = serverError;
const fs_1 = __importDefault(require("fs"));
/**
 * # Server Error
 */
function serverError(_a) {
    return __awaiter(this, arguments, void 0, function* ({ user, message, component, noMail, req, }) {
        const date = new Date();
        const reqIp = (() => {
            if (!req)
                return null;
            try {
                const forwarded = req.headers["x-forwarded-for"];
                const realIp = req.headers["x-real-ip"];
                const cloudflareIp = req.headers["cf-connecting-ip"];
                // Convert forwarded IPs to string and get the first IP if multiple exist
                const forwardedIp = Array.isArray(forwarded)
                    ? forwarded[0]
                    : forwarded === null || forwarded === void 0 ? void 0 : forwarded.split(",")[0];
                const clientIp = cloudflareIp ||
                    forwardedIp ||
                    realIp ||
                    req.socket.remoteAddress;
                if (!clientIp)
                    return null;
                return String(clientIp);
            }
            catch (error) {
                return null;
            }
        })();
        try {
            let log = `🚀 SERVER ERROR ===========================\nError Message: ${message}\nComponent: ${component}`;
            if ((user === null || user === void 0 ? void 0 : user.id) && (user === null || user === void 0 ? void 0 : user.first_name) && (user === null || user === void 0 ? void 0 : user.last_name) && (user === null || user === void 0 ? void 0 : user.email)) {
                log += `\nUser Id: ${user === null || user === void 0 ? void 0 : user.id}\nUser Name: ${user === null || user === void 0 ? void 0 : user.first_name} ${user === null || user === void 0 ? void 0 : user.last_name}\nUser Email: ${user === null || user === void 0 ? void 0 : user.email}`;
            }
            if (req === null || req === void 0 ? void 0 : req.url) {
                log += `\nURL: ${req.url}`;
            }
            if (req === null || req === void 0 ? void 0 : req.body) {
                log += `\nRequest Body: ${JSON.stringify(req.body, null, 4)}`;
            }
            if (reqIp) {
                log += `\nIP: ${reqIp}`;
            }
            log += `\nDate: ${date.toDateString()}`;
            log += "\n========================================";
            if (!fs_1.default.existsSync(`./.tmp/error.log`)) {
                fs_1.default.writeFileSync(`./.tmp/error.log`, "", "utf-8");
            }
            const initialText = fs_1.default.readFileSync(`./.tmp/error.log`, "utf-8");
            fs_1.default.writeFileSync(`./.tmp/error.log`, log);
            fs_1.default.appendFileSync(`./.tmp/error.log`, `\n\n\n\n\n${initialText}`);
        }
        catch ( /** @type {any} */error) {
            console.log("Server Error Reporting Error:", error.message);
        }
    });
}
////////////////////////////////////////
////////////////////////////////////////
////////////////////////////////////////
