"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = varReadOnlyDatabaseDbHandler;
const serverError_1 = __importDefault(require("./serverError"));
const parseDbResults_1 = __importDefault(require("./parseDbResults"));
const conn_db_handler_1 = __importDefault(require("../../utils/db/conn-db-handler"));
/**
 * # Read Only Db Handler with Varaibles
 * @returns
 */
function varReadOnlyDatabaseDbHandler(_a) {
    return __awaiter(this, arguments, void 0, function* ({ queryString, queryValuesArray, tableSchema, forceLocal, }) {
        var _b;
        let results;
        const DB_CONN = forceLocal
            ? global.DSQL_DB_CONN
            : global.DSQL_READ_ONLY_DB_CONN || global.DSQL_DB_CONN;
        try {
            results = yield (0, conn_db_handler_1.default)(DB_CONN, queryString, queryValuesArray);
        }
        catch (error) {
            (0, serverError_1.default)({
                component: "varReadOnlyDatabaseDbHandler",
                message: error.message,
                noMail: true,
            });
            (_b = global.ERROR_CALLBACK) === null || _b === void 0 ? void 0 : _b.call(global, `VarReadOnlyDB Handler Error`, error);
            return error.message;
        }
        finally {
            DB_CONN === null || DB_CONN === void 0 ? void 0 : DB_CONN.end();
        }
        if (results) {
            const unparsedResults = results;
            const parsedResults = yield (0, parseDbResults_1.default)({
                unparsedResults: unparsedResults,
                tableSchema: tableSchema,
            });
            return parsedResults;
        }
        else {
            return null;
        }
    });
}
